#ifndef VDG_H_INCLUDED
#define VDG_H_INCLUDED

#include "typedef.h"
#include "device.h"
#include "ini.h"
#include "vsurface.h"
#include "device/mc6847.h"


////////////////////////////////////////////////////////////////
// NX`
////////////////////////////////////////////////////////////////
class VDG6 : public Device, public VSurface, public virtual MC6847core, public IDoko {
protected:
	// J[R[h
	// for 60,61 -----------------------------------------------------------------------------
	static const BYTE COL60_AN[];
	static const BYTE COL60_SG[];
	static const BYTE COL60_CG[][8];
	static const BYTE COL60_RG[][2];
	// ---------------------------------------------------------------------------------------
	
	// for 62,66,64,68 -----------------------------------------------------------------------
	static const BYTE COL62_AN[];
	static const BYTE COL62_SG[];
	static const BYTE COL62_CG[][8];
	static const BYTE COL62_RG[][2];
	static const BYTE COL62_AN2[];
	static const BYTE COL62_CG2[][16];
	// ---------------------------------------------------------------------------------------
	
	WORD AddrOff;								// VRAM,ATTRItZbg
	
	// M[N
	bool VSYNC;									// tO
	bool HSYNC;									// tO
	int VLcnt;									// \CJE^
	int HSdclk;									// g[^(hbgNbN)
	int Hclk60;									// \(N60)
	
	bool CreateBuffer( int=1 );					// obNobt@쐬
	BYTE *GetBufAddr() const;					// obt@AhX擾
	int GetBufPitch() const;					// obt@sb`(1LineoCg)擾
	
	void LatchAttr();							// Agr[gf[^b`
	void LatchGMODE();							// Agr[gf[^b`(OtBbN[ĥ)
	BYTE GetAttr() const;						// Agr[gf[^擾
	virtual BYTE GetVram() const;				// VRAMf[^擾
	BYTE GetFont1( WORD ) const;				// Font1f[^擾
	BYTE GetFont2( WORD ) const;				// Font2f[^擾
	BYTE GetFont3( WORD ) const;				// Font3f[^擾
	
	virtual WORD GerVramAddr() const = 0;		// VRAMAhX擾
	virtual WORD GerAttrAddr() const = 0;		// ATTRAhX擾
	virtual void SetAttrAddr( BYTE );			// ATTRAhXݒ
	
	// I/OANZX֐
	void OutB0H( int, BYTE );
	
	// for 62,66,64,68 -----------------------------------------------------------------------
	void OutC0H( int, BYTE );
	void OutC1H( int, BYTE );
	BYTE InA2H( int );
	// ---------------------------------------------------------------------------------------
	
public:
	VDG6( VM6 *, const ID& );					// RXgN^
	virtual ~VDG6();							// fXgN^
	
	virtual void EventCallback( int, int );		// CxgR[obN֐
	
	virtual bool Init();						// 
	virtual void Reset();						// Zbg
	
	bool IsBusReqStop() const;					// oXNGXgԒ~tO擾
	bool IsBusReqExec() const;					// oXNGXgԎstO擾
	bool IsSRmode() const;						// SR[h擾
	bool IsSRGVramAccess( WORD ) const;			// SRG-VRAMANZX?
	WORD SRGVramAddr( WORD ) const;				// SRG-VRAMAhX擾
	
	// ------------------------------------------
	bool DokoSave( cIni * );	// ǂłSAVE
	bool DokoLoad( cIni * );	// ǂłLOAD
	// ------------------------------------------
};


class VDG60 : public VDG6, public MC6847 {
protected:
	WORD GerVramAddr() const;					// VRAMAhX擾
	WORD GerAttrAddr() const;					// ATTRAhX擾
	
	// foCX`
	static const Descriptor descriptor;
	static const InFuncPtr  indef[];
	static const OutFuncPtr outdef[];
	const Descriptor* GetDesc() const { return &descriptor; }
	
public:
	VDG60( VM6 *, const ID& );					// RXgN^
	virtual  ~VDG60();							// fXgN^
	
	// foCXID
	enum IDOut{ outB0H=0 };
	enum IDIn {};
};


class VDG62 : public VDG6, public virtual PCZ80_07 {
protected:
	WORD GerVramAddr() const;					// VRAMAhX擾
	WORD GerAttrAddr() const;					// ATTRAhX擾
	
	// foCX`
	static const Descriptor descriptor;
	static const InFuncPtr  indef[];
	static const OutFuncPtr outdef[];
	const Descriptor* GetDesc() const { return &descriptor; }
	
public:
	VDG62( VM6 *, const ID& );					// RXgN^
	virtual ~VDG62();							// fXgN^
	
	// foCXID
	enum IDOut{ outB0H=0, outC0H, outC1H };
	enum IDIn {  inA2H=0 };
};


class VDG64 : public VDG6, public PCZ80_12 {
protected:
	BYTE GetVram() const;						// VRAMf[^擾
	WORD GerVramAddr() const;					// VRAMAhX擾
	WORD GerAttrAddr() const;					// ATTRAhX擾
	void SetAttrAddr( BYTE );					// ATTRAhXݒ
	
	// foCX`
	static const Descriptor descriptor;
	static const InFuncPtr  indef[];
	static const OutFuncPtr outdef[];
	const Descriptor* GetDesc() const { return &descriptor; }
	
	// I/OANZX֐
	void Out4xH( int, BYTE );
	void OutC8H( int, BYTE );
	void OutC9H( int, BYTE );
	void OutCAH( int, BYTE );
	void OutCBH( int, BYTE );
	void OutCCH( int, BYTE );
	void OutCEH( int, BYTE );
	void OutCFH( int, BYTE );
	
public:
	VDG64( VM6 *, const ID& );					// RXgN^
	virtual ~VDG64();							// fXgN^
	
	void EventCallback( int, int );				// CxgR[obN֐
	
	void Reset();								// Zbg
	
	// foCXID
	enum IDOut{ out4xH=0, outB0H, outC0H, outC1H, outC8H, outC9H,
				outCAH,   outCBH, outCCH, outCEH, outCFH };
	enum IDIn {  inA2H=0 };
};


#endif	// VDG_H_INCLUDED
